shader_type canvas_item;
render_mode blend_premul_alpha,unshaded;

uniform sampler2D iChannel0;

// https://www.shadertoy.com/view/llfyz4

const float pi=3.1415926;


vec3 toYCbCr(vec3 rgb)
{
    return rgb*mat3(vec3(0.299,0.587,0.114),vec3(-0.168736,-0.331264,0.5),vec3(0.5,-0.418688,-0.081312))+vec3(0,.5,.5);
}


vec3 pre( vec2 coord, vec2 iResolution){
    return floor(256.*(toYCbCr((texture(iChannel0, coord/iResolution.xy).xyz)) - .5));
}

vec3 DCT8x8( vec2 coord, vec2 uv, vec2 iResolution ) {
    vec3 res = vec3(0);
    for(float x = 0.; x < 8.; x++){
    	for(float y = 0.; y < 8.; y++){
            res += pre(coord + vec2(x,y), iResolution) *
                cos((2.*x+1.)*uv.x*pi/16.) * 
                cos((2.*y+1.)*uv.y*pi/16.); 
    	}
    }
    return res * .25 * (uv.x!=0.?1.:1./sqrt(2.)) * (uv.y!=0.?1.:1./sqrt(2.));
}


void mainImage( out vec4 fragColor, in vec2 fragCoord, vec2 iResolution )
{
    vec2 mouse = 0.5*iResolution.xy/2.;
    int array[64]=int[64](
  16,  11,  10,  16,  24,  40,  51,  61,
  12,  12,  14,  19,  26,  58,  60,  55,
  14,  13,  16,  24,  40,  57,  69,  56,
  14,  17,  22,  29,  51,  87,  80,  62,
  18,  22,  37,  56,  68, 109, 103,  77,
  24,  35,  55,  64,  81, 104, 113,  92,
  49,  64,  78,  87, 103, 121, 120, 101,
  72,  92,  95,  98, 112, 100, 103,  99
);
    
    fragColor.w = 0.;
    vec2 uv = floor(fragCoord-8.*floor(fragCoord/8.));
    float q = (1.+mouse.y/iResolution.y*20.)*float(array[int(uv.x+uv.y*8.)]);
    fragColor.xyz = (floor(.5+DCT8x8(8.*floor(fragCoord/8.),uv,iResolution)/q))*q;
    
}

void fragment(){
    vec2 iResolution=1./TEXTURE_PIXEL_SIZE;
    mainImage(COLOR,FRAGCOORD.xy,iResolution);
}
